@@ -9,15 +9,17 @@ no warnings 'numeric';
 use Lock::Server;
 use Yote;
 
+use bytes;
+use IO::Socket::SSL;
 use JSON;
 use Time::HiRes qw(time);
 use URI::Escape;
 
 use vars qw($VERSION);
 
-$VERSION = '1.03';
+$VERSION = '1.04';
 
-my $DEBUG = 0;
+our $DEBUG = 1;
 
 sub new {
     my( $pkg, $args ) = @_;
@@ -40,17 +42,62 @@ sub new {
     }, $class;
 } #new
 
+sub store {
+    shift->{STORE};
+}
+
+sub load_options {
+
+    my( $yote_root_dir ) = @_;
+
+    my $confile = "$yote_root_dir/yote.conf";
+
+    #
+    # set up default options
+    #
+    my $options = {
+        yote_root_dir        => $yote_root_dir,
+        yote_host            => '127.0.0.1',
+        yote_port            => 8881,
+        lock_port            => 8004,
+        lock_host            => '127.0.0.1',
+        lock_attempt_timeout => 12,
+        lock_timeout         => 10,
+        use_ssl              => 0,
+        SSL_cert_file        => '',
+        SSL_key_file         => '',
+    };
+
+    #
+    # override base defaults with those from conf file
+    #
+    if( -f $confile && -r $confile ) {
+        # TODO - create conf with defaults and make it part of the install
+        open( IN, "<$confile" ) or die "Unable to open config file $@ $!";
+        while( <IN> ) {
+            chomp;
+            s/\#.*//;
+            if( /^\s*([^=\s]+)\s*=\s*([^\s].*)\s*$/ ) {
+                if( defined $options->{$1} ) {
+                    $options->{$1} = $2 if defined $options->{$1};
+                } else {
+                    print STDERR "Warning: encountered '$1' in file. Ignoring";
+                }
+            }
+        }
+        close IN;
+    } #if config file is there
+
+    return $options;
+}
+
+
 sub start {
     my $self = shift;
 
     $self->{STORE}{_locker} = $self->{_locker};
     $self->{_locker}->start;
 
-#    $self->{SERVER_ROOT} = $self->{STORE}->fetch_server_root;
-
-#    use Devel::SimpleProfiler;Devel::SimpleProfiler::start;
-    # child process
-
     my $listener_socket = $self->_create_listener_socket;
     die "Unable to open socket " unless $listener_socket;
 
@@ -94,29 +141,52 @@ sub run {
 sub _create_listener_socket {
     my $self = shift;
 
-    unless( $self->{STORE}{_locker} ) {
-        $self->{STORE}{_locker} = $self->{_locker};
-        $self->{_locker}->start;
-    }
-
     my $listener_socket;
     my $count = 0;
-    while( ! $listener_socket && ++$count < 10 ) { 
-        $listener_socket = new IO::Socket::INET(
-            Listen    => 10,
-            LocalAddr => "$self->{yote_host}:$self->{yote_port}",
-            );
+
+    if( $self->{use_ssl} && ( ! $self->{SSL_cert_file} || ! $self->{SSL_key_file} ) ) {
+        die "Cannot start server. SSL selected but is missing filename for SSL_cert_file and/or SSL_key_file";
+    }
+    while( ! $listener_socket && ++$count < 10 ) {
+        if( $self->{args}{use_ssl} ) {
+            my $cert_file = $self->{args}{SSL_cert_file};
+            my $key_file  = $self->{args}{SSL_key_file};
+            if( index( $cert_file, '/' ) != 0 ) {
+                $cert_file = "$self->{yote_root_dir}/$cert_file";
+            }
+            if( index( $key_file, '/' ) != 0 ) {
+                $key_file = "$self->{yote_root_dir}/$key_file";
+            }
+            $listener_socket = new IO::Socket::SSL(
+                Listen    => 10,
+                LocalAddr => "$self->{yote_host}:$self->{yote_port}",
+                SSL_cert_file => $cert_file,
+                SSL_key_file => $key_file,
+                );
+        } else {
+            $listener_socket = new IO::Socket::INET(
+                Listen    => 10,
+                LocalAddr => "$self->{yote_host}:$self->{yote_port}",
+                );
+        }
         last if $listener_socket;
         
-        print STDERR "Unable to open the yote socket. Retry $count of 10\n";
+        print STDERR "Unable to open the yote socket [$self->{yote_host}:$self->{yote_port}] ($!). Retry $count of 10\n";
         sleep 5 * $count;
     }
+
     unless( $listener_socket ) {
         $self->{error} = "Unable to open yote socket on port '$self->{yote_port}' : $! $@\n";
         $self->{_locker}->stop;
         _log( "unable to start yote server : $@ $!." );
         return 0;
     }
+
+    unless( $self->{STORE}{_locker} ) {
+        $self->{STORE}{_locker} = $self->{_locker};
+        $self->{_locker}->start;
+    }
+
     print STDERR "Starting yote server\n";
 
     unless( $self->{yote_root_dir} ) {
@@ -149,8 +219,9 @@ sub _run_loop {
 }
 
 sub _log {
-    my $msg = shift;
-    print STDERR "Yote::Server : $msg\n" if $DEBUG;
+    my( $msg, $sev ) = @_;
+    $sev //= 1;
+    print STDERR "Yote::Server : $msg\n" if $sev <= $DEBUG;
 }
 
 sub __transform_params {
@@ -218,16 +289,21 @@ sub _process_request {
         
         my $req = <$sock>;
         $ENV{REMOTE_HOST} = $sock->peerhost;
-        my %headers;
+        my( %headers, %cookies );
         while( my $hdr = <$sock> ) {
             $hdr =~ s/\s*$//s;
             last if $hdr !~ /[a-zA-Z]/;
             my( $key, $val ) = ( $hdr =~ /^([^:]+):(.*)/ );
             $headers{$key} = $val;
         }
-        my $store = $self->{STORE};
 
-        _log( "\n--> : $req" );
+        for my $cookie ( split( /\s*;\s*/, $headers{Cookie} ) ) {
+           $cookie =~ s/^\s*|^\s*$//g;
+            my( $key, $val ) = split( /\s*=\s*/, $cookie, 2 );
+            $cookies{ $key } = $val;
+        }
+        
+        _log( "\n[$$]--> : $req" );
 
         # 
         # read certain length from socket ( as many bytes as content length )
@@ -249,13 +325,13 @@ sub _process_request {
                 _log( "'$filename' exists" );
                 my @stat = stat $filename;
 
+                my $content_type = $filename =~ /css$/ ? 'text/css' : 'text/html';
                 my @headers = (
-                    'Content-Type: text/html; charset=utf-8',
+                    "Content-Type: $content_type; charset=utf-8",
                     'Server: Yote',
                     "Content-Length: $stat[7]",
                 );
 
-
                 open( IN, "<$filename" );
 
                 $sock->print( "HTTP/1.1 200 OK\n" . join ("\n", @headers). "\n\n" );
@@ -274,60 +350,18 @@ sub _process_request {
         
 
         # data has the input parmas in JSON format.
-        # GET /obj/action/params
-        # POST /obj/action  (params in POST data)
+        # POST /
 
-        # root is /_/
-        my( $obj_id, $token, $action, $params );
-
-        if ( $verb eq 'GET' ) {
-            ( $obj_id, $token, $action, my @params ) = split( '/', substr( $path, 1 ) );
-
-            $params = [ map { URI::Escape::uri_unescape($_) } @params ];
-            
-        } elsif ( $verb eq 'POST' ) {
-            ( $obj_id, $token, $action ) = split( '/', substr( $path, 1 ) );
-
-            $params = $data ? from_json( $data ) : []; # this has to be checked against is valid, yes
-        }
-        _log( "\n   (params)--> : ".Data::Dumper->Dump([$params]) );
-
-        if ( substr( $action, 0, 1 ) eq '_' ) {
-            _log( "Bad action (underscore) : '$action'" );
+        if ( $verb ne 'POST' ) {
+            _log( "Attempted not-suppored '$verb' request" );
             $sock->print( "HTTP/1.1 400 BAD REQUEST\n\n" );
             $sock->close;
-            exit;
         }
 
-        my $server_root = $self->{STORE}->fetch_server_root;
-        my $server_root_id = $server_root->{ID};
-        unless( $obj_id eq '_' || 
-                    $obj_id eq $server_root_id || 
-                    ( $obj_id > 0 && 
-                      $server_root->_valid_token( $token, $ENV{REMOTE_HOST} ) && 
-                      $server_root->_getMay( $obj_id, $token ) ) ) {
-
-            # tried to do an action on an object it wasn't handed. do a 404
-            _log( "Bad Path : '$path'" );
-            $sock->print( "HTTP/1.1 400 BAD REQUEST\n\n" );
-            $sock->close;
-            exit;
-        }
-
-        if ( $params && ref( $params ) ne 'ARRAY' ) {
-            _log( "Bad Req Param Not Array : $params" );
-            $sock->print( "HTTP/1.1 400 BAD REQUEST\n\n" );
-            $sock->close;
-            exit;
-        }
-
-        # now things are getting a bit more complicated. The params passed in
-        # are always a list, but they may contain other containers that are not
-        # yote objects. So, transform the incomming parameter list and check all
-        # yote objects inside for may. Use a recursive helper function for this.
-        my $in_params;
+        $data =~ s/^p=//;
+        my $out_json;
         eval {
-            $in_params = $self->__transform_params( $params, $token, $server_root );
+            $out_json = $self->invoke_payload( $data );
         };
         if( $@ ) {
             _log( $@ );
@@ -335,129 +369,156 @@ sub _process_request {
             $sock->close;
             exit;
         }
-
-        my $obj = $obj_id eq '_' ? $server_root :
-            $store->fetch( $obj_id );
-        unless( $obj->can( $action ) ) {
-            _log( "Bad Req : invalid method :'$action'" );
-            $sock->print( "HTTP/1.1 400 BAD REQUEST\n\n" );
-            $sock->close;
-            exit;
-        }
-
-        my( @res );
-        eval {
-            if ( $action eq 'fetch' && $obj == $server_root ) {
-                # fetch is a special action that can return any 
-                # object in the system. It must check the token
-                # to see if that particular object is allowed/available
-                # to the caller
-                unshift( @$in_params, $token );
-            }
-            (@res) = ($obj->$action( @$in_params ));
-            if( $action eq 'init_root' ) {
-                my $token = $server_root->create_token;
-                push @res, $token;
-            }
-        };
-
-        if ( $@ ) {
-            _log( "INTERNAL SERVER ERROR '$@'" );
-            $sock->print( "HTTP/1.1 500 INTERNAL SERVER ERROR\n\n" );
-            $sock->close;
-            exit; #end forked process
-        }
-
-        my $out_res = $store->_xform_in( \@res, 'allow datastructures' );
-
-
-        my @has = _find_ids_in_data( $out_res );
-        my @mays = @has, $store->_find_ids_referenced( @has );
-
-        my $ids_to_update;
-        if ( ( $action eq 'fetch_root' || $action eq 'init_root' )  && ( $obj_id eq '_' || $obj_id eq $server_root_id ) ) {
-            # if there is a token, make it known that the token 
-            # has received server root data
-            $ids_to_update = [ $server_root_id ];
-            if ( $token > 1  ) {
-                unless( $store->_last_updated( $server_root_id ) ) {
-                    $store->{OBJ_UPDATE_DB}->put_record( $server_root_id, [ Time::HiRes::time ] );
-                }
-                push @has, $server_root_id;
-            }
-        } else {
-            $ids_to_update = $server_root->_updates_needed( $token, \@has );
-        }
-        
-        my( @updates, %methods );
-        for my $obj_id (@$ids_to_update) {
-            my $obj = $store->fetch( $obj_id );
-            my $ref = ref( $obj );
-
-            my( $data );
-            if ( $ref eq 'ARRAY' ) {
-                $data = [ 
-                    map { my $d = $store->_xform_in( $_ );
-                          push @mays, $d;
-                          $d } 
-                        @$obj ];
-            } elsif ( $ref eq 'HASH' ) {
-                $data = {
-                    map { my $d = $store->_xform_in( $obj->{$_} );
-                          push @mays, $d;
-                          $_ => $d }
-                        keys %$obj };
-            } else {
-                my $obj_data = $obj->{DATA};
-
-                $data = {
-                    map { my $d = $obj_data->{$_};
-                          push @mays, $d;
-                          $_ => $d }
-                        grep { $_ !~ /^_/ }
-                        keys %$obj_data };
-                $methods{$ref} ||= $obj->_callable_methods;
-            }
-            push @has, $obj_id;
-            my $update = {
-                id    => $obj_id,
-                cls   => $ref,
-                data  => $data,
-            };
-            push @updates, $update;
-        } #each obj_id to update
-
-        $server_root->_setHasAndMay( \@has, \@mays, $token );
-
-        my $out_json = to_json( { result  => $out_res,
-                                 updates => \@updates,
-                                 methods => \%methods,
-                             } );
         my @headers = (
             'Content-Type: text/json; charset=utf-8',
             'Server: Yote',
             'Access-Control-Allow-Headers: accept, content-type, cookie, origin, connection, cache-control',
             'Access-Control-Allow-Origin: *', #TODO - have this configurable
-            'Content-Length: ' . length( $out_json ),
-        );
-        _log( "<-- 200 OK ( " . join( ",", @headers ) . " ) ( $out_json )\n" );
+            'Content-Length: ' . bytes::length( $out_json ),
+            );
+        
+        _log( "<-- 200 OK [$$] ( " . join( ",", @headers ) . " ) ( $out_json )\n" );
         $sock->print( "HTTP/1.1 200 OK\n" . join ("\n", @headers). "\n\n$out_json\n" );
-
+        
         $sock->close;
         $self->{STORE}->stow_all;
         exit;
+
     } #child
 } #_process_request
 
+sub invoke_payload {
+    my $self     = shift;
+
+    _log( "jsony $_[0]" );
+
+    my $req_data = from_json( shift );
+
+    my( $obj_id, $token, $action, $params ) = @$req_data{ 'i', 't', 'a', 'pl' };
+
+    _log( "\n   (params [$$])--> : ".Data::Dumper->Dump([$params]) );
+    my $server_root = $self->{STORE}->fetch_server_root;
+    
+    my $server_root_id = $server_root->{ID};
+    my $session = $server_root->_fetch_session( $token );
+
+    unless( $obj_id eq '_' || 
+            $obj_id eq $server_root_id || 
+            ( $obj_id > 0 && 
+              $session && 
+              $server_root->_getMay( $obj_id, $session->get__token ) ) ) {
+        
+        # tried to do an action on an object it wasn't handed. do a 404
+        die( "Bad Path" );
+    }
+    if( substr( $action, 0, 1 ) eq '_' ) {
+        die( "Private method called" );
+    }
+
+    if ( $params && ref( $params ) ne 'ARRAY' ) {
+        die( "Bad Req Param Not Array : $params" );
+    }
+
+    # now things are getting a bit more complicated. The params passed in
+    # are always a list, but they may contain other containers that are not
+    # yote objects. So, transform the incomming parameter list and check all
+    # yote objects inside for may. Use a recursive helper function for this.
+
+    my $in_params = $self->__transform_params( $params, $token, $server_root );
+
+    my $store = $self->{STORE};
+
+    my $obj = $obj_id eq '_' ? $server_root :
+        $store->fetch( $obj_id );
+
+
+
+    unless( $obj->can( $action ) ) {
+        die( "Bad Req : invalid method :'$action'" );
+    }
+
+    if( $session ) {
+        $obj->{SESSION} = $session;
+        $obj->{SESSION}{SERVER_ROOT} = $server_root;
+    }
+
+    my(@res) = ($obj->$action( @$in_params ));
+    delete $obj->{SESSION};
+        
+    my $out_res = $store->_xform_in( \@res, 'allow datastructures' );
+
+    my @has = _find_ids_in_data( $out_res );
+    my @mays = @has, $store->_find_ids_referenced( @has );
+
+    my $ids_to_update;
+    if ( ( $action eq 'fetch_root' || $action eq 'init_root' || $action eq 'fetch_app' )  && ( $obj_id eq '_' || $obj_id eq $server_root_id ) ) {
+        # if there is a token, make it known that the token 
+        # has received server root data
+        $ids_to_update = [ $server_root_id, grep { $_ ne $server_root_id } @has ];
+        if ( $token  ) {
+            push @has, $server_root_id;
+        }
+    } else {
+        $ids_to_update = $server_root->_updates_needed( $token, \@has );
+    }
+    
+    my( @updates, %methods );
+    for my $obj_id (@$ids_to_update) {
+        my $obj = $store->fetch( $obj_id );
+        my $ref = ref( $obj );
+        
+        my( $data );
+        if ( $ref eq 'ARRAY' ) {
+            $data = [ 
+                map { my $d = $store->_xform_in( $_ );
+                      push @mays, $d;
+                      $d } 
+                @$obj ];
+        } elsif ( $ref eq 'HASH' ) {
+            $data = {
+                map { my $d = $store->_xform_in( $obj->{$_} );
+                      push @mays, $d;
+                      $_ => $d }
+                keys %$obj };
+        } else {
+            my $obj_data = $obj->{DATA};
+            
+            $data = {
+                map { my $d = $obj_data->{$_};
+                      push @mays, $d;
+                      $_ => $d }
+                grep { $_ !~ /^_/ }
+                keys %$obj_data };
+            $methods{$ref} ||= $obj->_callable_methods;
+        }
+        push @has, $obj_id;
+        my $update = {
+            id    => $obj_id,
+            cls   => $ref,
+            data  => $data,
+        };
+        push @updates, $update;
+    } #each obj_id to update
+
+    $server_root->_setHasAndMay( \@has, \@mays, $token );
+
+    my $out_json = to_json( { result  => $out_res,
+                              updates => \@updates,
+                              methods => \%methods,
+                            } );
+    return $out_json;
+} #invoke_payload
+
 # ------- END Yote::Server
 
 package Yote::ServerStore;
 
-use strict;
-use warnings;
-no warnings 'uninitialized';
+use vars qw($VERSION);
 
-use DB::DataStore;
+$VERSION = '1.0';
+
+
+use Data::RecordStore;
 
 use base 'Yote::ObjStore';
 
@@ -469,7 +530,8 @@ sub _new { #Yote::ServerStore
     # keeps track of when any object had been last updated.
     # use like $self->{OBJ_UPDATE_DB}->put_record( $obj_id, [ time ] );
     # or my( $time ) = @{ $self->{OBJ_UPDATE_DB}->get_record( $obj_id ) };
-    $self->{OBJ_UPDATE_DB} = DB::DataStore::FixedStore->open( "L", "$args->{root}/OBJ_META" );
+    $self->{OBJ_UPDATE_DB} = Data::RecordStore::FixedStore->open( "L", "$args->{root}/OBJ_META" );
+    $self->{OBJ_UPDATE_DB}->put_record( $self->{ID}, [ Time::HiRes::time ] );
     $self;
 } #_new
 
@@ -571,8 +633,6 @@ sub fetch_server_root {
     # or make it simple. if the webapp has an account, then pass that account
     # with the rest of the arguments
 
-    # verify the token - ip match in the server root object
-    
     # then verify if the command can run on the app object with those args
     # or even : $myapp->run( 'command', @args );
 
@@ -598,6 +658,11 @@ sub unlock {
 
 package Yote::ServerObj;
 
+use vars qw($VERSION);
+
+$VERSION = '1.0';
+
+
 use base 'Yote::Obj';
 
 sub _log {
@@ -615,7 +680,7 @@ sub __discover_methods {
     no strict 'refs';
     my @m = grep { $_ !~ /::/ } keys %{"${pkg}\::"};
     if( $pkg eq 'Yote::ServerObj' ) { #the base, presumably
-        return [ grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|AUTOLOAD|DESTROY|CARP_TRACE|BEGIN|isa|PKG2METHS|ISA)$)/ } @m ];
+        return [ grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|VERSION|AUTOLOAD|DESTROY|CARP_TRACE|BEGIN|isa|PKG2METHS|ISA)$)/ } @m ];
     }
 
     my %hasm = map { $_ => 1 } @m;
@@ -627,7 +692,7 @@ sub __discover_methods {
     
     my $base_meths = __discover_methods( 'Yote::ServerObj' );
     my( %base ) = map { $_ => 1 } 'AUTOLOAD', @$base_meths;
-    $meths = [ grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|AUTOLOAD|DESTROY|BEGIN|isa|PKG2METHS|ISA)$)/ && ! $base{$_} } @m ];
+    $meths = [ grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|VERSION|AUTOLOAD|DESTROY|BEGIN|isa|PKG2METHS|ISA)$)/ && ! $base{$_} } @m ];
     $Yote::ServerObj::PKG2METHS->{$pkg} = $meths;
     
     $meths;
@@ -681,19 +746,17 @@ sub set {
 
 package Yote::ServerRoot;
 
-use strict;
-use warnings;
-no warnings 'uninitialized';
+use vars qw($VERSION);
+
+$VERSION = '1.00';
 
 use base 'Yote::ServerObj';
 
 sub _init {
     my $self = shift;
-    $self->set__token2ip({});
     $self->set__doesHave_Token2objs({});
     $self->set__mayHave_Token2objs({});
     $self->set__apps({});
-    $self->set__appOnOff({});
     $self->set__token_timeslots([]);
     $self->set__token_timeslots_metadata([]);
     $self->set__token_mutex([]);
@@ -703,27 +766,28 @@ sub _log {
     Yote::Server::_log(shift);
 }
 
-sub _valid_token {
-    my( $self, $token, $ip ) = @_;
+sub _fetch_session {
+    my( $self, $token ) = @_;
     my $slots = $self->get__token_timeslots();
     for( my $i=0; $i<@$slots; $i++ ) {
-        if( $slots->[$i]{$token} eq $ip ) {
+        if( my $session = $slots->[$i]{$token} ) {
             if( $i < $#$slots ) {
-                # refresh its time
-                $slots->[0]{ $token } = $ip;
+                # make sure this is in the most current 'boat'
+                $slots->[0]{ $token } = $session;
             }
-            return 1;
+            return $session;
         }
     }
-    0;
-}
+    
+} #_fetch_sesion
 
 
 sub _resetHasAndMay {
-    my( $self, $tokens ) = @_;
+    my( $self, $tokens, $doesHaveOnly ) = @_;
 
     $self->{STORE}->lock( "_has_and_may_Token2objs" );
-    for ( qw( doesHave mayHave ) ) {
+    my( @lists ) = $doesHaveOnly ? qw( doesHave ) : qw( doesHave mayHave );
+    for ( @lists ) {
         my $item = "_${_}_Token2objs";
         my $token2objs = $self->_get( $item );
         for my $token (@$tokens) {
@@ -790,8 +854,11 @@ sub _updates_needed {
     \@updates;
 } #_updates_needed
 
-
 sub create_token {
+    shift->_create_session->get__token;
+}
+
+sub _create_session {
     my $self = shift;
     my $tries = shift;
 
@@ -799,8 +866,7 @@ sub create_token {
         die "Error creating token. Got the same random number 4 times in a row";
     }
 
-    my $randpart = int( rand( 1_000_000_000 ) ); #TODO - find max this can be for long int
-    my $ip = $ENV{REMOTE_HOST};
+    my $token = int( rand( 1_000_000_000 ) ); #TODO - find max this can be for long int
     
     # make the token boat. tokens last at least 10 mins, so quantize
     # 10 minutes via time 10 min = 600 seconds = 600
@@ -828,26 +894,29 @@ sub create_token {
     # If already used, try this again :/
     #
     for( my $i=0; $i<@$slot_data; $i++ ) {
-        return $self->create_token( ++$tries ) if $slots->[ $i ]{ $randpart };
+        return $self->_create_session( $tries++ ) if $slots->[ $i ]{ $token };
     }
 
     #
-    # See if this belongs on the most recent boat. If not, then
+    # See if the most recent time slot is current. If it is behind, create a new current slot
     # create a new most recent boat.
     #
+    my $session = $self->{STORE}->newobj( { 
+        _token => $token } );
     if( $slot_data->[ 0 ] == $current_time_chunk ) {
-        $slots->[ 0 ]{ $randpart } = $ip;
+        $slots->[ 0 ]{ $token } = $session;
     } else {
         unshift @$slot_data, $current_time_chunk;
-        unshift @$slots, { $randpart => $ip };
+        unshift @$slots, { $token => $session };
     }
+    
 
     #
     # remove this token from old boats so it doesn't get purged
     # when in a valid boat.
     #
     for( my $i=1; $i<@$slot_data; $i++ ) {
-        delete $slots->[$i]{ $randpart };
+        delete $slots->[$i]{ $token };
     }
 
     #
@@ -869,32 +938,28 @@ sub create_token {
     }
     
     $self->{STORE}->unlock( 'token_mutex' );
-#    print STDERR Data::Dumper->Dump([$slot_data,$slots,"MADE TOKEN $randpart"]);
-#
-# $VAR1 = [
-#           '14575528'
-#         ];
-# $VAR2 = [
-#           {
-#             '913149795' => '127.0.0.1'
-#           }
-#         ];
-#
-#
     
-    return $randpart;
+    return $session;
 
-} #create_token
+} #_create_session
+
+sub _destroy_session {
+    my( $self, $token ) = @_;
+    my $slots = $self->get__token_timeslots();
+    for( my $i=0; $i<@$slots; $i++ ) {
+        delete $slots->[$i]{ $token };
+    }
+    print STDERR Data::Dumper->Dump(["DESTROY '$token'",$slots]);
+
+    $self->_resetHasAndMay( [$token] );
+    1;
+} #_destroy_session
 
 #
-# what things will the server root provide?
-# logins? apps?
+# Returns the app and possibly a logged in account
 #
-# fetch_app? It's just an object.
-#
-
 sub fetch_app {
-    my( $self, $app_name, @args ) = @_;
+    my( $self, $app_name ) = @_;
     my $apps = $self->get__apps;
     my $app  = $apps->{$app_name};
     unless( $app ) {
@@ -909,12 +974,8 @@ sub fetch_app {
         $app = $app_name->_new( $self->{STORE} );
         $apps->{$app_name} = $app;
     }
-    my $appIsOn = $self->get__appOnOff->{$app_name};
-    if( $appIsOn eq 'off' ) {
-        _log( "App '$app_name' not found" );
-        return undef;
-    }
-    return $app->_can_access( @args ) ? $app : undef;
+    
+    return $app, $self->{SESSION} ? $self->{SESSION}->get_acct : undef;
 } #fetch_app
 
 sub fetch_root {
@@ -922,11 +983,18 @@ sub fetch_root {
 }
 
 sub init_root {
-    return shift;
+    my $self = shift;
+    my $session = $self->{SESSION} || $self->_create_session;
+    my $token = $session->get__token;
+    $self->_resetHasAndMay( [ $token ], 'doesHaveOnly' );
+    return $self, $token;
 }
 
 sub fetch {
-    my( $self, $token, @ids ) = @_;
+    my( $self, @ids ) = @_;
+    
+    $self->{SESSION} && ( my $token = $self->{SESSION}->get__token ) || return;
+    
     my $mays = $self->get__mayHave_Token2objs;
     my $may = $self->get__mayHave_Token2objs()->{$token};
     my $store = $self->{STORE};
@@ -946,25 +1014,21 @@ sub update {
 
 # ------- END Yote::ServerRoot
 
-package Yote::ServerApp;
-
-
-use strict;
-use warnings;
-no warnings 'uninitialized';
-
-use base 'Yote::ServerObj';
-
-sub _can_access {
-    1;
-}
-
-# ------- END Yote::ServerApp
-
 1;
 
 __END__
 
+=head1 NAME
+
+Yote::Server - Serve up marshaled perl objects in javascript
+
+=head1 DESCRIPTION
+
+=cut
+
+
+
+
 
 okey, this is going to have something like
 
